AC_DEFUN([AX_PATH_LIB_NET_SNMP],[dnl
AC_MSG_CHECKING([for Net-SNMP])
NETSNMP_CFLAGS=`net-snmp-config --cflags`
NETSNMP_LIBS=`net-snmp-config --libs`
if test ".$NETSNMP_CFLAGS" == "." ; then
    AC_MSG_ERROR([Could not find Net-SNMP headers])
fi
if test ".$NETSNMP_LIBS" == "." ; then
    AC_MSG_ERROR([Could not find Net-SNMP libraries])
fi
AC_MSG_RESULT(yes)
OLDCFLAGS="$CFLAGS"
OLDLDFLAGS="$LDFLAGS"
CFLAGS="$CFLAGS $NETSNMP_CFLAGS"
LDFLAGS="$LDFLAGS $NETSNMP_LIBS"
AC_CHECK_FUNC(snmp_sess_init,[
    AC_SUBST(NETSNMP_CFLAGS)
    AC_SUBST(NETSNMP_LIBS)
    JUNK="junk"
    dnl AC_MSG_RESULT(yes)
],[
    AC_MSG_ERROR([no. Check "Problems Compiling" in the docs.])
])
CFLAGS="$OLDCFLAGS"
LDFLAGS="$OLDLDFLAGS"
])



AC_DEFUN([AX_FUNC_NEW_RRD_GRAPH],[dnl
AC_MSG_CHECKING([whether this is the new rrd_graph])
AC_CACHE_VAL(ac_cv_new_rrd_graph,
[AC_COMPILE_IFELSE([
#include <stdio.h>
#include <rrd.h>
int main (int argc, char** argv)
{
    int chk;
    int paramCnt;
    char** params;
    char** results;
    int w;
    int h;
    double j1;
    double j2;

    chk = rrd_graph ( paramCnt, params, &results, &w, &h, NULL, &j1, &j2 );
    exit(0);
}], ac_cv_new_rrd_graph=yes, ac_cv_new_rrd_graph=no)]
AC_MSG_RESULT($ac_cv_new_rrd_graph)
)])



AC_DEFUN([AX_FUNC_OLD_RRD_GRAPH],[dnl
AC_MSG_CHECKING([whether this is the old rrd_graph])
AC_CACHE_VAL(ac_cv_old_rrd_graph,
[AC_COMPILE_IFELSE([
#include <stdio.h>
#include <rrd.h>
int main (int argc, char** argv)
{
    int chk;
    int paramCnt;
    char** params;
    char** results;
    int w, h;

    chk = rrd_graph ( paramCnt, params, &results, &w, &h );
    exit(0);
}],ac_cv_old_rrd_graph=yes,ac_cv_old_rrd_graph=no)]
AC_MSG_RESULT($ac_cv_old_rrd_graph)
)])



AC_DEFUN([AX_PATH_LIB_RRD],[dnl
AC_MSG_CHECKING([for librrd])
AC_ARG_WITH(rrd,
            AS_HELP_STRING([--with-rrd(=PREFIX)], [use rrd (default is yes)]),
            [WITH_RRD=$withval],
            [WITH_RRD=yes])
if test ".${WITH_RRD}" = ".no" ; then
    RRD_CFLAGS=""
    RRD_LDFLAGS=""
    AC_DEFINE_UNQUOTED(WITH_RRD, 0, [use rrdtool])
else
    if test ".${WITH_RRD}" = ".yes" ; then
        AC_MSG_RESULT(yes)
        AC_MSG_CHECKING(-lrrd)
        AC_CHECK_LIB(rrd,rrd_create,[
            RRD_CFLAGS=""
            RRD_LDFLAGS="-lrrd"
            dnl AC_MSG_RESULT(yes)
        ],[
            AC_MSG_ERROR([librrd not found. Make sure it is installed. If the
            header and library files are not in a standard location use
            --with-rrd=PREFIX.])
        ])
    else
        if test -d "${WITH_RRD}" ; then
            OLDCFLAGS="$CFLAGS"
            OLDLDFLAGS="$LDFLAGS"
            CFLAGS="$CFLAGS -I${WITH_RRD}/include"
            LDFLAGS="$LDFLAGS -L${WITH_RRD}/lib -lrrd"
            AC_MSG_RESULT(${WITH_RRD})
            AC_CHECK_LIB(rrd, rrd_create,[
                RRD_CFLAGS="-I${WITH_RRD}/include"
                RRD_LDFLAGS="-L${WITH_RRD}/lib -lrrd"
                dnl AC_MSG_RESULT(yes)
            ],[
                AC_MSG_ERROR([librrd not found. Looks like the PREFIX you specified
                is wrong.])
            ])
            CFLAGS="$OLDCFLAGS"
            LDFLAGS="$OLDLDFLAGS"
        else
            AC_MSG_ERROR([PREFIX is not a directory. It should be the prefix
            directory of your RRDTool.])
        fi
    fi
    OLDCFLAGS="$CFLAGS"
    OLDLDFLAGS="$LDFLAGS"
    CFLAGS="$CFLAGS $RRD_CFLAGS"
    LDFLAGS="$LDFLAGS $RRD_LDFLAGS"
    AX_FUNC_NEW_RRD_GRAPH
    if test ".$ac_cv_new_rrd_graph" = ".yes" ; then
        AC_DEFINE(NEW_RRD_GRAPH, 1, [librrd uses the new signature for rrd_graph])
    else
        AX_FUNC_OLD_RRD_GRAPH
        if test ".$ac_cv_old_rrd_graph" = ".yes" ; then
            AC_DEFINE(NEW_RRD_GRAPH, 0, [librrd uses the old signature for rrd_graph])
        else
            AC_MSG_ERROR([Neither interface for rrd_graph worked. Giving up!])
        fi
    fi
    CFLAGS="$OLDCFLAGS"
    LDFLAGS="$OLDLDFLAGS"
    AC_DEFINE_UNQUOTED(WITH_RRD, 1, [use rrdtool])
fi
# Write and exit
AC_SUBST(RRD_LDFLAGS)
AC_SUBST(RRD_CFLAGS)
])



AC_DEFUN([AX_OPT_RRDDIR],[dnl
AC_MSG_CHECKING([for default rrd directory])
APPEND=/var/snmp4nagios/rrd
AC_ARG_WITH(rrddir,
    AS_HELP_STRING([--with-rrddir=DIR],
                   [default RRDDIR (PREFIX/var/snmp4nagios/rrd)]),
    [
        test ".$withval" = ".yes" \
        && {
            test ".$prefix" = ".NONE"  \
            && RRDDIR=${ac_default_prefix}${APPEND} \
            || RRDDIR=${prefix}${APPEND}
        } || {
            test ".$withval" = ".no" \
            && {
                test ".$prefix" = ".NONE"  \
                && RRDDIR=${ac_default_prefix}${APPEND} \
                || RRDDIR=${prefix}${APPEND}
            } || {
                RRDDIR=$withval
            }
        }
    ],[
        test ".$prefix" = ".NONE"  \
        && RRDDIR=${ac_default_prefix}${APPEND} \
        || RRDDIR=${prefix}${APPEND}
    ])
AC_DEFINE_UNQUOTED(RRDDIR, "$RRDDIR", [path where rrd files are put by default])
AC_MSG_RESULT($RRDDIR)
])



AC_DEFUN([AX_OPT_PNGDIR],[dnl
AC_MSG_CHECKING([for default png directory])
APPEND=/var/snmp4nagios/png
AC_ARG_WITH(pngdir,
    AS_HELP_STRING([--with-pngdir=DIR],
                   [default PNGDIR (PREFIX/var/snmp4nagios/png)]),
    [
        test ".$withval" = ".yes" \
        && {
            test ".$prefix" = ".NONE"  \
            && PNGDIR=${ac_default_prefix}${APPEND} \
            || PNGDIR=${prefix}${APPEND}
        } || {
            test ".$withval" = ".no" \
            && {
                test ".$prefix" = ".NONE"  \
                && PNGDIR=${ac_default_prefix}${APPEND} \
                || PNGDIR=${prefix}${APPEND}
            } || {
                PNGDIR=$withval
            }
        }
    ],[
        test ".$prefix" = ".NONE"  \
        && PNGDIR=${ac_default_prefix}${APPEND} \
        || PNGDIR=${prefix}${APPEND}
    ])
AC_DEFINE_UNQUOTED(PNGDIR, "$PNGDIR", [path where png files are put by default])
AC_MSG_RESULT($PNGDIR)
])
