/*
 * File:      check_cpq_fcahctrl.c
 * Copyright: (c) 2006 by Peter Gritsch
 * Email:     s4nag@no-where.at
 *
 * This file is part of SNMP4Nagios.
 *
 * SNMP4Nagios is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * SNMP4Nagios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SNMP4Nagios; if not, write to the
 *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA
 */

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <errno.h>
#include "globals.h"
#include "nagiosif.h"
#include "snmpif.h"
#include "syslogif.h"
#include "strutils.h"
#include "utilities.h"


/* data type for storing scan results */
typedef struct
{
    int index;   /* host controller index */
    int v2;      /* found cpqFcaHostCntlrSlot */
    int v3;      /* found cpqFcaHostCntlrModel */
    int v4;      /* found cpqFcaHostCntlrStatus */
    int v5;      /* found cpqFcaHostCntlrCondition */
    int slot;    /* slot index */
    char* model; /* model name */
} tScanRes;

/* data type for storing data */
typedef struct
{
    int   stat;  /* status */
    int   cond;  /* condition */
} tDataSet;


/*
 * SNMP base OID
 * CPQFCA-MIB::cpqFcaHostCntlrIndex.IDX
 * = .1.3.6.1.4.1.232.16.2.7.1.1.1.IDX
 * CPQFCA-MIB::cpqFcaHostCntlrSlot.IDX
 * = .1.3.6.1.4.1.232.16.2.7.1.1.2.IDX
 * CPQFCA-MIB::cpqFcaHostCntlrModel.IDX
 * = .1.3.6.1.4.1.232.16.2.7.1.1.3.IDX
 * CPQFCA-MIB::cpqFcaHostCntlrStatus.IDX
 * = .1.3.6.1.4.1.232.16.2.7.1.1.4.IDX
 * CPQFCA-MIB::cpqFcaHostCntlrCondition.IDX
 * = .1.3.6.1.4.1.232.16.2.7.1.1.5.IDX
 */
oid baseOID [] = { 1, 3, 6, 1, 4, 1, 232, 16, 2, 7, 1, 1 };
size_t baseOIDLen = 12;

/* scan results */
tScanRes* scanRes = NULL;
int scanResLen = 0;

/* results of the SNMP queries */
tDataSet v;


void SetGlobals ()
{
    /* progName is set by ReadArgs() */
    srvName = "check_cpq_fcahctrl.c";
    progVersion = "0.2";
    helpIndexP = "-i INDEX";
    helpThresP = NULL;
    helpPurpose =
            "Checks the status of a  Compaq/HP"
            "Fibre Channel Host Controller.";
    /* community is set by ReadArgs() */
    /* commFile is set by ReadArgs() */
    /* rrdStep is unused */
    /* rrdHBeat is unused */
    supportsPerfLog = FALSE;
    hasIndexParam = TRUE;
}

void HelpSpecificParams ()
{
    printf (
    "-i, --index INTEGER\n"
    "    Index of the host controller.\n"
    "\n"
    );
}

void HelpExplain ()
{
    printf (
    "The plugin returns the state of a host controller.\n"
    "\n"
    );
}

void HelpPerformanceData ()
{
    printf (
    "There is no performance data provided.\n"
    "\n"
    );
}

int ChkOptSpecificParams ( int argc, char** argv, int* i )
{
    return FALSE;
}

char* DecodeCPQFhcModel ( int aType )
{
    char* retVal = NULL;

    switch ( aType )
    {
        case  1:
            retVal = StrDup ( "other" );
        break;
        case  2:
            retVal = StrDup ( "Compaq StorageWorks Fibre Channel Host Bus"
                    " Adapter/P." );
        break;
        case  3:
            retVal = StrDup ( "Compaq StorageWorks Fibre Channel Host Bus"
                    " Adapter/E." );
        break;
        case  4:
            retVal = StrDup ( "Compaq StorageWorks 64-Bit/66-Mhz Fibre Host"
                    " Bus Adapter." );
        break;
        case  5:
            retVal = StrDup ( "Compaq Smart Array, SAN Access Module." );
        break;
        case  6:
            retVal = StrDup ( "FCA-2101" );
        break;
        case  7:
            retVal = StrDup ( "Compaq StorageWorks 64bit/33Mhz PCI to Fibre"
                    " Channel HBA." );
        break;
        case  8:
            retVal = StrDup ( "FCA-221x" );
        break;
        case  9:
            retVal = StrDup ( "Dual Port Fibre Channel Mezzanine Card (2 Gb)"
                    " for BL20Gp2 G2." );
        break;
        case 10:
            retVal = StrDup ( "PCI-X 2Gb FCA2404 Fibre Channel HBA" );
        break;
        case 11:
            retVal = StrDup ( "PCI-X 2Gb FCA2214 Fibre Channel HBA" );
        break;
        case 12:
            retVal = StrDup ( "PCI-X 2Gb A7298A Fibre Channel HBA" );
        break;
        case 13:
            retVal = StrDup ( "PCI-X 2Gb FCA2214DC Fibre Channel HBA" );
        break;
        case 14:
            retVal = StrDup ( "PCI-X Dual Channel 2GB A6826A Fibre Channel"
                    " HBA" );
        break;
        case 15:
            retVal = StrDup ( "Fibre Channel Mezzaine Card G3, BL3x p"
                    " Series" );
        break;
        case 16:
            retVal = StrDup ( "Fibre Channel Mezzaine Card G4, BL2x p"
                    " Series" );
        break;
        case 17:
            retVal = StrDup ( "PCI-X 2GB AB466A/AB467A Fibre Channel HBA" );
        break;
        default:
            retVal = StrDup ( "invalid" );
        break;
    }
    return retVal;
}

int ScanResIndex ( int aIndex )
{
    int retVal = 0;
    int found = FALSE;

    while ( ( retVal < scanResLen ) && !found )
    {
        found = ( scanRes [retVal].index == aIndex );
        if ( !found ) retVal++;
    }
    if ( !found ) retVal = -1;
    return retVal;
}

void SnmpWalkHandleVariable ( netsnmp_variable_list* aVar )
{
    if ( aVar->name_length == 14 )
    {
        if ( OidCmp ( aVar->name, baseOID, baseOIDLen ) == 0 )
        {
            int index;

            switch ( aVar->name [12] )
            {
                case 1: /* cpqFcaHostCntlrIndex */
                    index = scanResLen;
                    scanResLen++;
                    scanRes = realloc ( scanRes,
                            sizeof ( tScanRes ) * scanResLen );
                    if ( scanRes == NULL ) ErrorMalloc ();
                    scanRes [index].index = aVar->name [13];
                    scanRes [index].v2 = FALSE;
                    scanRes [index].v3 = FALSE;
                    scanRes [index].v4 = FALSE;
                    scanRes [index].v5 = FALSE;
                    scanRes [index].model = NULL;
                    scanRes [index].slot = -1;
                break;
                case 2: /* cpqFcaHostCntlrSlot */
                    index = ScanResIndex ( aVar->name [13] );
                    if ( ( aVar->type == ASN_INTEGER ) )
                    {
                        scanRes [index].v2 = ( aVar->type == ASN_INTEGER );
                        scanRes [index].slot = *(aVar->val.integer);
                    } /* else nothing to do */
                break;
                case 3: /* cpqFcaHostCntlrModel */
                    index = ScanResIndex ( aVar->name [13] );
                    if ( ( aVar->type == ASN_INTEGER ) )
                    {
                        scanRes [index].v3 = TRUE;
                        scanRes [index].model =
                                DecodeCPQFhcModel ( *(aVar->val.integer) );
                    } /* else nothing to do */
                break;
                case 4: /* cpqFcaHostCntlrStatus */
                    index = ScanResIndex ( aVar->name [13] );
                    scanRes [index].v4 = ( aVar->type == ASN_INTEGER );
                break;
                case 5: /* cpqFcaHostCntlrCondition */
                    index = ScanResIndex ( aVar->name [13] );
                    scanRes [index].v5 = ( aVar->type == ASN_INTEGER );
                break;
                default:
                    /* don't care */
                break;
            }
        } else Log ( LOG_CRIT, "Unexpected OID while scanning services" );
    } else Log ( LOG_CRIT, "Wrong length while scanning services" );
}

int ScanServices ()
{
    int i;
    int retVal = STATE_UNKNOWN;

    SnmpWalk ( baseOID, baseOIDLen );
    for ( i = 0; i < scanResLen; i++ )
    {
        if (    scanRes [i].v2 && scanRes [i].v3 && scanRes [i].v4 &&
                scanRes [i].v5 )
        {
            retVal = STATE_OK;
            printf ( "OK index: %i ; slot: %i ; model: %s\n",
                    scanRes [i].index, scanRes [i].slot, scanRes [i].model );
            free ( scanRes [i].model );
        }
    }
    if ( retVal == STATE_UNKNOWN ) printf ( "FAILED\n" );
    return retVal;
}

void ReadVals ()
{
    struct snmp_session sess1;
    struct snmp_session* sess2;
    oid o_stat [14];
    oid o_cond [14];
    size_t oidLen;

    OidCpy ( o_stat, &oidLen, baseOID, &baseOIDLen );
    OidCpy ( o_cond, &oidLen, baseOID, &baseOIDLen );
    o_stat [12] = 4; o_stat [13] = idx;
    o_cond [12] = 5; o_cond [13] = idx;
    oidLen = 14;
    ConfigureSession1 ( &sess1 );
    sess2 = snmp_open ( &sess1 );
    if ( sess2 == NULL ) Error ( LOG_ERR, "Could not open session!" );
    v.stat = ReadInteger ( sess2, o_stat, oidLen );
    v.cond = ReadInteger ( sess2, o_cond, oidLen );
    free ( sess1.peername );
    free ( sess1.community );
    snmp_close_sessions ();
}

void CreateDataFile ( const char* aRRDFPath )
{
    /* performance data is not supported */
}

void ParseData ( const char* aData )
{
    /* performance data is not supported */
}

void WriteData ( const char* aRRDFPath )
{
    /* performance data is not supported */
}

int CheckData ()
{
    int retVal = STATE_OK;
    char* cond = NULL;
    char* stat = NULL;

    ReadVals ();
    switch ( v.cond )
    {
        case 1:  retVal = STATE_WARNING;  cond = "'other'";    break;
        case 2:  retVal = STATE_OK;       cond = "'ok'";       break;
        case 3:  retVal = STATE_WARNING;  cond = "'degraded'"; break;
        case 4:  retVal = STATE_CRITICAL; cond = "'failed'";   break;
        default: retVal = STATE_UNKNOWN;  cond = "invalid";    break;
    }
    switch ( v.stat )
    {
        case 1:
            retVal = MAXIMUM ( retVal, STATE_WARNING);
            stat = "'other'";
        break;
        case 2:
            stat = "'ok'";
        break;
        case 3:
            retVal = MAXIMUM ( retVal, STATE_CRITICAL);
            stat = "'failed'";
        break;
        case 4:
            retVal = MAXIMUM ( retVal, STATE_CRITICAL);
            stat = "'shutdown'";
        break;
        case 5:
            retVal = MAXIMUM ( retVal, STATE_WARNING);
            stat = "'loop degraded'";
        break;
        case 6:
            retVal = MAXIMUM ( retVal, STATE_CRITICAL);
            stat = "'loop failed'";
        break;
        default:
            retVal = MAXIMUM ( retVal, STATE_UNKNOWN);
            stat = "invalid";
        break;
    }
    PrintExit ( retVal );
    printf ( " - Condition is %s, state is %s\n", cond, stat );
    return ( retVal );
}

int main ( int argc, char** argv )
{
    int retVal = STATE_OK;

    SetGlobals ();
    ReadArgs ( argc, argv );
    if ( scanServices ) retVal = ScanServices ();
    else
    {
        if ( checkData ) retVal = CheckData ();
    }
    if ( hostName != NULL ) free ( hostName );
    if ( hostAddr != NULL ) free ( hostAddr );
    if ( progName != NULL ) free ( progName );
    return ( retVal );
}
