/*
 * File:      nagiosif.c
 * Copyright: (c) 2006 by Peter Gritsch
 * Email:     s4nag@no-where.at
 *
 * This file is part of SNMP4Nagios.
 *
 * SNMP4Nagios is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * SNMP4Nagios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SNMP4Nagios; if not, write to the
 *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA
 */

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include "nagiosif.h"
#include "syslogif.h"
#include "strutils.h"


char* GetEnv ( char* aVar, int aFail )
{
    char* val = getenv ( aVar );
    if ( !val && aFail )
    {
        char msg[1024];
        snprintf ( msg, 1024,
                "Could not read environment variable %s.\n", aVar );
        Error ( LOG_ERR, msg );
    }
    return val;
}

char* GetData ()
{
    return StrDup ( GetEnv ( "NAGIOS_SERVICEPERFDATA", TRUE ) );
}

void ReadHostName ()
{
    char* name = GetEnv ( "NAGIOS_HOSTNAME", FALSE );

    if ( name != NULL )
    {
        hostName = StrDup ( name );
        isHostNameSet = TRUE;
    } /* else the defaults will do. */
}

void ReadHostAddr ()
{
    char* addr = GetEnv ( "NAGIOS_HOSTADDRESS", FALSE );

    if ( addr != NULL )
    {
        hostAddr = StrDup ( addr );
        isHostAddrSet = TRUE;
    } /* else the defaults will do. */
}

#if(WITH_RRD)
char* GetRRDHostDirPath ()
{
    char* hostDir;

    hostDir = StrConcat ( 3, rrdDir, "/", hostName );
    return hostDir;
}

char* GetPNGHostDirName ()
{
    char* hostDir;

    hostDir = StrConcat ( 3, pngDir, "/", hostName );
    return hostDir;
}

ulong GetTimestamp ()
{
    /*
    ulong timestamp;
    char* raw = GetEnv ( "NAGIOS_LASTSERVICECHECK", TRUE );
    char* tail = raw;

    errno = 0;
    timestamp = strtoul ( raw, &tail, 0 );
    if ( errno || *tail )
            Error ( LOG_ERR,
            "Invalid timestamp $NAGIOS_LASTSERVICECHECK" );
    return timestamp;
    */
    return time ( NULL );
}
#endif /* WITH_RRD */

void PrintExit ( int aRetVal )
{
    switch ( aRetVal )
    {
        case STATE_OK:        printf ( "OK" );        break;
        case STATE_WARNING:   printf ( "WARNING" );   break;
        case STATE_CRITICAL:  printf ( "CRITICAL" );  break;
        case STATE_UNKNOWN:   printf ( "UNKNOWN" );   break;
        case STATE_DEPENDENT: printf ( "DEPENDENT" ); break;
        default:              printf ( "INVALID" );   break;
    }
}
