/*
 * File:      nagiosif.h
 * Copyright: (c) 2006 by Peter Gritsch
 * Email:     s4nag@no-where.at
 *
 * This file is part of SNMP4Nagios.
 *
 * SNMP4Nagios is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * SNMP4Nagios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SNMP4Nagios; if not, write to the
 *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA
 */

#ifndef NAGIOSIF_H
#define NAGIOSIF_H


#include "globals.h"


/**********************/
/* Exported constants */

/*
 * Exit codes
 */
#define STATE_OK        0
#define STATE_WARNING   1
#define STATE_CRITICAL  2
#define STATE_UNKNOWN   3
#define STATE_DEPENDENT 4


/**********************/
/* Exported functions */

/*
 * Reads the performance data from the environment variable
 * "$NAGIOS_SERVICEPERFDATA" and returns it in a newly malloced buffer
 * which should be freed by the caller.
 */
char* GetData ();

/*
 * Tries to read the host name from the environment variable
 * "$NAGIOS_HOSTNAME". If it succeeds, then a new buffer is allocated (which
 * should be freed some time) and the variable isHostNameSet is set to TRUE.
 */
void ReadHostName ();

/*
 * Tries to read the host name from the environment variable
 * "$NAGIOS_HOSTADDRESS". If it succeeds, then a new buffer is allocated (which
 * should be freed some time) and the variable isHostNameSet is set to TRUE.
 */
void ReadHostAddr ();

#if(WITH_RRD)
/*
 * Reads the hostname from the environment variable "$NAGIOS_HOSTNAME",
 * builds the path to the directory where the rrd data files for this host
 * are stored and returns it in a newly malloced buffer which should be
 * freed by the caller.
 *
 * The basename for the rrd archives is either given with the -R option
 * or the compiled in default.
 */
char* GetRRDHostDirPath ();

/*
 * Reads the hostname from the environment variable "$NAGIOS_HOSTNAME",
 * builds the name of the directory where the png plot files for this host
 * are stored and returns it in a newly malloced buffer which should be
 * freed by the caller.
 *
 * The basename for the png files is either given with the -G option or the
 * compiled in default.
 */
char* GetPNGHostDirName ();

/*
 * This returns the current time from the system. It used to read the
 * timestamp of the sample from the environment variable
 * "$NAGIOS_LASTSERVICECHECK" which made ugly results (hundreds of "unknown"
 * services) when nagios was restarted.
 */
ulong GetTimestamp ();
#endif /* WITH_RRD */

/*
 * Prints the appropriate string (without newline) for the given return
 * value.
 *
 * aRetVal Value the program will return.
 */
void PrintExit ( int aRetVal );

#endif /* NAGIOSIF_H */
