/*
 * File:      rrdif.h
 * Copyright: (c) 2006 by Peter Gritsch
 * Email:     s4nag@no-where.at
 *
 * This file is part of SNMP4Nagios.
 *
 * SNMP4Nagios is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * SNMP4Nagios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SNMP4Nagios; if not, write to the
 *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA
 */

#ifndef RRDIF_H
#define RRDIF_H

#if(WITH_RRD)


/**********************/
/* Exported constants */

/*
 * types of data sources
 */
#define GAUGE    1
#define COUNTER  2
#define DERIVE   3
#define ABSOLUTE 4

/*
 * consolidation functions
 */
#define MIN 9
#define MAX 10
#define AVG 11 /* AVERAGE */
#define LST 12 /* LAST */

/*
 * predefined RRAs
 */
#define RRA_MAX_7H_1M   CreateRRAStr ( MAX,       7,    1 )
#define RRA_AVG_7H_1M   CreateRRAStr ( AVG,       7,    1 )
#define RRA_MIN_7H_1M   CreateRRAStr ( MIN,       7,    1 )
#define RRA_MAX_25H_5M  CreateRRAStr ( MAX,      25,    5 )
#define RRA_AVG_25H_5M  CreateRRAStr ( AVG,      25,    5 )
#define RRA_MIN_25H_5M  CreateRRAStr ( MIN,      25,    5 )
#define RRA_MAX_8D_15M  CreateRRAStr ( MAX,    8*24,   15 )
#define RRA_AVG_8D_15M  CreateRRAStr ( AVG,    8*24,   15 )
#define RRA_MIN_8D_15M  CreateRRAStr ( MIN,    8*24,   15 )
#define RRA_MAX_32D_1H  CreateRRAStr ( MAX,   32*24,   60 )
#define RRA_AVG_32D_1H  CreateRRAStr ( AVG,   32*24,   60 )
#define RRA_MIN_32D_1H  CreateRRAStr ( MIN,   32*24,   60 )
#define RRA_MAX_367D_4H CreateRRAStr ( MAX,  367*24, 4*60 )
#define RRA_AVG_367D_4H CreateRRAStr ( AVG,  367*24, 4*60 )
#define RRA_MIN_367D_4H CreateRRAStr ( MIN,  367*24, 4*60 )

/*
 * predefined timings
 */
#define PLOT_1H_10S   17
#define PLOT_6H_1M    18
#define PLOT_24H_5M   19
#define PLOT_7D_15M   20
#define PLOT_31D_60M  21
#define PLOT_366D_4H  22

/*
 * vertical sizes of plots
 */
#define PLOT_H_UNIDIR 200
#define PLOT_H_BIDIR  320

/*
 * SI exponent names
 */
#define PLOT_SI_AUTO   33
#define PLOT_SI_FEMTO  34
#define PLOT_SI_PICO   35
#define PLOT_SI_NANO   36
#define PLOT_SI_MICRO  37
#define PLOT_SI_MILLI  38
#define PLOT_SI_NONE   39
#define PLOT_SI_KILO   40
#define PLOT_SI_MEGA   41
#define PLOT_SI_GIGA   42
#define PLOT_SI_TERA   43
#define PLOT_SI_PETA   44

/*
 * Plot types
 */
#define PLOT_AREA   65
#define PLOT_LINE1  66
#define PLOT_LINE2  67
#define PLOT_LINE3  68
#define PLOT_STACK  69

/*
 * Predefined colors
 */
#define C_USAGE1_MAX "#ffbbbb"
#define C_USAGE1_AVG "#f83838"
#define C_USAGE2_MAX "#ffda9f"
#define C_USAGE2_AVG "#ff9900"
#define C_FREE_MAX   "#ccffcc"
#define C_FREE_AVG   "#3bf63b"
#define C_LOAD1_MAX  "#ff0000"
#define C_LOAD1_AVG  "#9f0000"
#define C_LOAD2_MAX  "#00ff00"
#define C_LOAD2_AVG  "#009f00"
#define C_LOAD3_MAX  "#0000ff"
#define C_LOAD3_AVG  "#00009f"
#define C_LOAD4_MAX  "#ffff00"
#define C_LOAD4_AVG  "#9f9f00"
#define C_THRU_MAX   "#ccccff"
#define C_THRU_AVG   "#3b3bf6"
#define C_ERR1_MAX   "#ffbbbb"
#define C_ERR1_AVG   "#f83838"
#define C_ERR2_MAX   "#ffda9f"
#define C_ERR2_AVG   "#ff9900"
#define C_THRES_MAX  "#ffbbbb"
#define C_THRES_AVG  "#f83838"
#define C_GAUGE1_MAX "#ccccff"
#define C_GAUGE1_AVG "#3b3bf6"



/******************/
/* Exported types */

/*
 * File Names structure.
 *
 * Used to store various file names and pathes, handled by the FN* functions.
 */
typedef struct
{
    char* rrdHostDirPath;
    char* rrdFilePath;
    char* pngHostDirName;
    char* pngFileName;
    char* pngFilePath;
    char* tmpPNGFilePath;
} tFN;

/*
 * Plot Parameters structure.
 *
 * Used to handle various parameters of a plot like unit, dimension and
 * padding of texts. Handled by the PP* functions
 */
typedef struct
{
    char** dim;
    char** unit;
    int* stack;
    int count;
    int dimLen;
    int unitLen;
} tPP;


/**********************/
/* Exported functions */

/*
 * Returns the range in seconds of the given timing.
 */
int TimingGetRange ( int aTiming );

/*
 * Returns the sample interval in seconds of the given timing.
 */
int TimingGetInterval ( int aTiming );

/*
 * Returns the width scale factor of the given timing.
 */
float TimingGetXScale ( int aTiming );

/*
 * Returns the description string for the given timing. This string is
 * newly malloced and should be freed by the caller.
 */
char* TimingGetDescr ( int aTiming );

/*
 * Checks if a directory exists. If not then it tries to create it. If the
 * directory neither existed nor could be created, then it exits the
 * program.
 */
void CheckDir ( const char* aDir );

/*
 * Builds a filename from the host directory and the globals srvName and
 * srvId. If srvId is NULL, then the name is build without it. The function
 * returns the file name in a newly malloced buffer which should be freed by
 * the caller.
 */
char* BuildRRDFilePath ( const char* aRRDHostDirPath );

/*
 * Checks if a rrd data file exists. If not then it tries to create it by
 * calling CreateDataFile(). If it the database still does not exist, then
 * it exits the program.
 */
void CheckRRDDataFile ( const char* dataFile );

/*
 * Creates a string suitable as "DS" term for "rrd create".
 *
 * param aName name of the data source.
 * param aType type ot the data source, see above and man 1 rrdcreate
 * param aHBeat maximum time in seconds between samples.
 * param aMin string containing the minimum for values of this DS
 * param aMax string containing the maximum for values of this DS
 *
 * return pointer to a newly malloced string containing the term
 *
 * For more information about the parameters, see man 1 rrdcreate
 */
char* CreateDSStr ( const char* aName, int aType, int aHBeat, char* aMin,
        char* aMax );

/*
 * Creates a string suitable as "RRA" term for "rrd create".
 *
 * param aCF consolidation function
 * param aRangeHours number of hours this RRA should keep
 * param aCDPIntMinutes number of minutes between Consolidated Data Points
 *
 * return pointer to a newly malloced string containing the term
 */
char* CreateRRAStr ( int aCF, int aRangeHours, int aCDPIntMinutes );

/*
 * Creates a rrd archive by calling rrd_create and evaluates the return value.
 *
 * param aParamCnt number of parameters in the aParams array.
 * param aParams the params array.
 * param aRRDFilePath the rrd file from which the plot is created.
 */
void RRDCreate ( int aParamCnt, char** aParams, const char* aRRDFilePath );

/*
 * Updates a rrd archive -- that is: writes values to that archive -- by
 * calling rrd_update and evaluates the return value.
 *
 * param aParamCnt number of parameters in the aParams array.
 * param aParams the params array.
 * param aRRDFilePath the rrd file from which the plot is created.
 */
void RRDUpdate ( int aParamCnt, char** aParams, const char* aRRDFilePath );

/*
 * Initializes a tFN structure and returns a pointer to it. This structure has
 * to be modified by FNSetPNGData() and can then be used to create a plot.
 * Once finished, FNDone() should be called to free all resources used
 * by the structure.
 *
 * return pointer to a newly malloced tFN structure.
 */
tFN* FNInit ();

/*
 * Releases all resources allocated by the given tFN structure and the
 * structure itself.
 *
 * param aFN Structure which should be deleted.
 */
void FNDone ( tFN* aFN );

/*
 * Adds information about the plot file to the given tFN structure.
 *
 * param aFN Structure which should be modified.
 * param aPlotId the plot id (e.g. "pkt" or "oct"), can be 0.
 * param aTiming timing from which a description string is constructed.
 */
void FNSetPNGData ( tFN* aFN, const char* aPlotId, int aTiming );

/*
 * Deletes information about a specific plot from the given tFN structure.
 * This can be called after a plot file has been created. It will be called
 * by FNSetPNGData() anyway.
 */
void FNUnsetPNGData ( tFN* aFN );

/*
 * Initializes a tPP structure and returns a pointer to it. This structure has
 * contains a set of data for each set of parameters which should be printed
 * using PlotPrintParams(). Once finished, PPDone() should be called to free
 * all resources used by the structure.
 *
 * return pointer to a newly malloced tPP structure.
 */
tPP* PPInit ();

/*
 * Adds information about one type of data (roughly: one call to
 * PlotPrintParams()). Note that "plots" must be be added in the same order
 * the corresponding calls to PlotPrintParams() are made, otherwise the
 * descriptions will get messed up!
 *
 * param aPP Structure which should be modified.
 * param aDim Dimension of the data (e.g. "usage", "width", "errors/time")
 * param aUnit Unit of the data (e.g. "%", "mm", "1/s" or "errors/s")
 * param aStack TRUE if the plot is stacked, false otherwise. Note: This does
 *         not make the plot stacked, it only changes the text of some output.
 */
void PPAddPlot ( tPP* aPP, const char* aDim, const char* aUnit, int aStack );

/*
 * Releases all resources allocated by the given tPP structure and the
 * structure itself.
 *
 * param aPP Structure which should be deleted.
 */
void PPDone ( tPP* aPP );

/*
 * Sets up the standard parameters for plotting data from a RRD file.
 * All strings are newly allocated and should be freed by the caller.
 * If plotNewOutput is TRUE then the title and the start and end times
 * are written to stdout.
 *
 * param array of strings in which the parameters are returned.
 * param aFN tFN structure for this plot
 * param aTitle title of the graph
 * param aTiming timing given as one of the predefined timing constants.
 * param aVLabel Y-Unit of the graph
 * param aEnd Time at which the last plot should end. Should be a rrd
 *         heartbeat ago.
 * param aSIBase Base for SI Multipliers (1024 or 1000).
 * param aSIExp SI Exponent as defined by "SI exponent names" above.
 * param aLower lower limit.
 * param aUpper upper limit. Upper and lower limit are only used if
 *         aLower < aUpper. Otherwise rrdtool's autoconfiguration is used.
 * param aUsingBidir not null if the plot uses negative number. Just to
 *         make the plot larger.
 *
 * return the number of parameters assigned so far.
 */
int PlotStdParams ( char** aParams, tFN* aFN, const char* aTitle,
        const char* aVLabel, int aTiming, int aEnd, int aSIBase, int aSIExp,
        int aLower, int aUpper, int aUsingBidir );

/*
 * Creates a definition for a virtual name of a data source.
 *
 * param aVName Virtual name of the data source.
 * param aFN tFN structure for this plot
 * param aDS The name of the data source.
 * param aCF The consolidation function as given in rrdif.h.
 *
 * return pointer to a newly malloced string containing the term.
 */
char* CreateDEF ( const char* aVName, tFN* aFN, const char* aDS,
        int aCF );

/*
 * Creates a definition for a plot.
 *
 * param aType Type of the plot as defined in rrd.h
 * param aVName Virtual name of the data source.
 * param aColor Web-like color specification for the line. This can be NULL
 *         in which case the plot will be blind and no legend will be printed.
 * param aPP Plot parameters structure for this plot.
 * param aPPI Index of the plot parameters starting at 0.
 * param aCF The consolidation function as given in rrdif.h. This will be
 *         printed right of the name (if that is not NULL).
 * param aNL Insert a new line at the end of the legend. In that case the
 *         legend will not be padded.
 *
 * return pointer to a newly malloced string containing the term.
 */
char* CreatePlot ( int aType, const char* aVName, const char* aColor,
        tPP* aPP, int aPPI, int aCF, int aNL );

/*
 * Creates the parameters for printing the average, minimum, maximum and last
 * values either into the plot itself (if plotNewOutputFormat is FALSE) or
 * to the results list (if plotNewOutputFormat is TRUE).
 *
 * Note: Padding will fail if the width of the format is too small.
 *
 * param aI Index of the first param to be set.
 * param aParams the params array.
 * param aPP Plot parameters structure for this plot.
 * param aPPI Index of the plot parameters starting at 0.
 * param aVNAvg Virtual name of the data source for the average value.
 * param aVNMin Virtual name of the data source for the minimum value.
 * param aVNMax Virtual name of the data source for the maximum value.
 * param aVNLst Virtual name of the data source for the last value.
 * param aFormat Format of the value (e.g. "%5.2lf").
 *
 * return Index of the next param to be set.
 */
int PlotPrintParams ( int aI, char** aParams, tPP* aPP, int aPPI,
        const char* aVNAvg, const char* aVNMin, const char* aVNMax,
        const char* aVNLst, const char* aFormat );

/*
 * Graphs a RRD archive by calling rrd_graph and evaluating the return
 * value and results array.
 *
 * param aParamCnt number of parameters in the aParams array.
 * param aParams the params array.
 * param aFN tFN structure for this plot.
 * param aPP tPP structure for this plot.
 */
void RRDGraph ( int aParamCnt, char** aParams, tFN* aFN, tPP* aPP );

/*
 * Logs the performance data.
 */
void LogPerfData ();

#endif /* WITH_RRD */

#endif /* RRDIF_H */
