/*
 * File:      common.h
 * Copyright: (c) 2006 by Peter Gritsch
 * Email:     s4nag@no-where.at
 *
 * This file is part of SNMP4Nagios.
 *
 * SNMP4Nagios is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * SNMP4Nagios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SNMP4Nagios; if not, write to the
 *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA
 */

#ifndef SNMPIF_H
#define SNMPIF_H


#ifndef NETSNMP_NO_INLINE
#define NETSNMP_NO_INLINE /* Turn off inlining */
#endif


#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>

/*
 * Tries to walk the tree below aRootOid and calls SnmpWalkHandleVariable()
 * for each variable it finds.
 */
void SnmpWalk ( oid aRootOid[], size_t aRootLen );

/*
 * Tries to configure the community string as described in utilities.c/
 * HelpCommunity(). Note that "free ( aSess->community );" should be
 * called as the community string is malloced by this function.
 */
void ConfigureCommunity ( struct snmp_session* aSess );

/*
 * Sets up a "first session". Use the source Luke!
 */
void ConfigureSession1 ( struct snmp_session* aSess );

/*
 * Converts an OID to its string representation.
 *
 * @param aBuffer Buffer where the string is placed.
 * @param aBufLen Length of the buffer. Note that the function will silently
 *         stop writing at the end of the buffer so better make that big
 *         enough.
 * @param aOid OID which is to convert.
 * @param aOidLen Length of the OID.
 */
void OIDToString ( char* aBuffer, int aBufLen, oid aOid[], size_t aOidLen);

/*
 * Compares two OIDs up to the length aLen.
 *
 * @param aOid1 First OID.
 * @param aOid2 Second OID.
 * @param aLen Length of the OIDs which are to compare
 * @return 0 if the OIDs are equal, not zero otherwise.
 */
int OidCmp ( oid aOid1[], oid aOid2[], size_t aLen );

/*
 * Copies aSrcLen oid elements from aSrcOid to aDstOid.
 *
 * @param aDstOid Destination OID. This must be at least aSrcLen long.
 * @param aDstLen Pointer to the length of the destination OID. This is
 *         used to return the new length of aDstOid.
 * @param aSrcOid Source OID.
 * @param aSrcLen Length of the Source OID (or the part which is to copy).
 */
void OidCpy ( oid aDstOid[], size_t* aDstLen, oid aSrcOid[], size_t* aSrcLen );

/*
 * Read an Integer, Counter32 or Gauge from the snmp agent.
 *
 * @param aSess Session which is to be used for accessing the agent.
 * @param aOid OID of the variable which is to be read.
 * @param aOidLen Length of the OID.
 *
 * @return The contents of the variable.
 */
int ReadInteger ( struct snmp_session* aSess, oid aOid[], size_t aOidLen );
uint ReadCounter32 ( struct snmp_session* aSess, oid aOid[], size_t aOidLen );
uint ReadGauge32 ( struct snmp_session* aSess, oid aOid[], size_t aOidLen );

/*
 * Copies a string from a netsnmp_variable_list. This is useful because
 * netsnmp_variable_lists contain strings which are not null terminated.
 */
char* VarListStrDup ( netsnmp_variable_list* aVar );

/*
 * Returns a newly malloced string which should be freed.
 */
char* ReadString ( struct snmp_session* aSess, oid aOid[], size_t aOidLen );

#endif /* SNMPIF_H */
