/*
 * File:      strutils.h
 * Copyright: (c) 2006 by Peter Gritsch
 * Email:     s4nag@no-where.at
 *
 * This file is part of SNMP4Nagios.
 *
 * SNMP4Nagios is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * SNMP4Nagios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SNMP4Nagios; if not, write to the
 *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA
 */

#ifndef STRUTILS_H
#define STRUTILS_H


/**********************/
/* Exported functions */

/*
 * Duplicates a string by calling strdup() and checking the result. If an
 * error occured, it exits the program.
 *
 * param aStr string which should be duplicated.
 *
 * return pointer to the new string which should be freed by the caller.
 */
char* StrDup ( const char* aStr );

/*
 * Converts a long integer to its decimal string representation. If an error
 * occured, it exits the program.
 *
 * param aInt long int which should be converted.
 *
 * return pointer to the new string which should be freed by the caller.
 */
char* IntToStr ( long aInt );

/*
 * Converts an unsigned long integer to its decimal string representation. If
 * an error occured, it exits the program.
 *
 * param aUInt unsigned int which should be converted.
 *
 * return pointer to the new string which should be freed by the caller.
 */
char* UIntToStr ( unsigned long aUInt );

/*
 * Converts a double to its decimal string representation. If an error
 * occured, it exits the program.
 *
 * param aDouble double which should be converted.
 *
 * return pointer to the new string which should be freed by the caller.
 */
char* DoubleToStr ( double aDouble );

/*
 * Concatenates a variable number of strings and returns the resulting string
 * in a newly allocated string. If an error occured, it exits the program.
 *
 * param aCnt First string.
 * param ... variable arguments which are treated like "const char*"s.
 *
 * return pointer to the new string which should be freed by the caller.
 */
char* StrConcat ( uint aCnt, ... );

/*
 * Checks if a string follows the "ident" rule, that is consists only of
 * letters (A-Z, a-z), digits (0-9) a/o the underscore (_). An empty string
 * does not follow the rule.
 *
 * param aIdent Pointer to the string which should be checked.
 * return TRUE if the ident follows the rule.
 */
int CheckIdent ( const char* aIdent );

/*
 * Returns the last component of a file path in a newly allocated string.
 * - It does not modify aFilename.
 * - It never returns any slash.
 * - It returns an empty string ("") if aFilename == NULL or aFilename is ""
 *   or aFilename consists only of slashes.
 * - It returns "." if aFilename is "."
 *
 * param aFilename file path of which to find the last component.
 * return pointer to the new string which should be freed by the caller.
 */
char* Basename ( const char *aFilename );

#endif /* STRUTILS_H */
