/*
 * File:      syslogif.c
 * Copyright: (c) 2006 by Peter Gritsch
 * Email:     s4nag@no-where.at
 *
 * This file is part of SNMP4Nagios.
 *
 * SNMP4Nagios is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * SNMP4Nagios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SNMP4Nagios; if not, write to the
 *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA
 */

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h> /* rrd.h needs stdio.h to be included first (type FILE) */
#if(WITH_RRD)
#include <rrd.h>
#endif /* WITH_RRD */
#include <stdlib.h>
#include <syslog.h>
#include "globals.h"
#include "nagiosif.h"
#include "syslogif.h"


int DoLog ( int aPriority )
{
    return ( aPriority <= loggingLevel );
}

void Log ( int aPriority, char* aMsg )
{
    if ( DoLog ( aPriority ) )
    {
        openlog ( NULL, LOG_CONS | LOG_PID | LOG_NDELAY, LOGFACILITY);
        syslog ( aPriority, "%s", aMsg );
        closelog ();
    }
}

void LogParams ( int argv, char** argc )
{
    if ( DoLog ( LOG_DEBUG ) )
    {
        int i;
        openlog ( NULL, LOG_CONS | LOG_PID | LOG_NDELAY, LOGFACILITY);
        for ( i = 0; i < argv; i++ )
        {
            syslog ( LOG_DEBUG, "params[%i] = %s\n", i, argc [i] );
        }
        closelog ();
    }
}

void Error ( int aPriority, char* aMsg )
{
    Log ( aPriority, aMsg );
    printf ( "%s\n", aMsg );
    exit ( STATE_UNKNOWN );
}

void ErrorMalloc ()
{
    Error ( LOG_ERR, "Could not allocate memory.\n" );
}

#if(WITH_RRD)
void CheckRRDError ( int aChk, const char* aAction,
        const char* aDataFile )
{
    if ( aChk != 0 )
    {
        char msg[1024];
        snprintf ( msg, 1024,
                "Could not %s rrd file %s (chk = %i): %s\n",
                aAction, aDataFile, aChk, rrd_get_error () );
        Error ( LOG_ERR, msg );
    }
}
#endif /* WITH_RRD */
