/*
 * File:      syslogif.h
 * Copyright: (c) 2006 by Peter Gritsch
 * Email:     s4nag@no-where.at
 *
 * This file is part of SNMP4Nagios.
 *
 * SNMP4Nagios is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * SNMP4Nagios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SNMP4Nagios; if not, write to the
 *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA
 */

#ifndef SYSLOGIF_H
#define SYSLOGIF_H


/**********************/
/* Exported functions */

/*
 * returns not 0 if the system wide logging level is at least the given
 * logging level aLevel. That is: it returns true if something that has
 * the logging priority aLevel should be logged. (Note: "at least" is
 * given with respect to the priority i.e. emergency is more than "debug".
 * The actual values of those macros are sorted reversely.
 */
int DoLog ( int aPriority );

/*
 * helper function which opens an appropriate system log and writes the
 * message aMsg if aPriority is equal or higher than the system wide logging
 * level.
 */
void Log ( int aPriority, char* aMsg );

/*
 * helper function which logs the parameters for a rrd_* function call if
 * the system wide logging level is LOG_DEBUG.
 */
void LogParams ( int argv, char** argc );

/*
 * helper function which calls Log() and exits with STATE_UNKNOWN.
 */
void Error ( int aPriority, char* aMsg );

/*
 * Writes "Could not allocate memory." to the log and exits.
 */
void ErrorMalloc ();

#if(WITH_RRD)
/*
 * helper function which checks if aChk is not equal 0 ( rrd_*() returns -1
 * on error). If so, it logs an error message ("Could not <aAction> rrd file
 * <aDataFile> (chk = <aChk>: <rrd_get_error()>") and exits with the return
 * code aExitCode.
 */
void CheckRRDError ( int aChk, const char* aAction, const char* aDataFile );
#endif /* WITH_RRD */

#endif /* SYSLOGIF_H */
