/*
 * File:      utilities.h
 * Copyright: (c) 2006 by Peter Gritsch
 * Email:     s4nag@no-where.at
 *
 * This file is part of SNMP4Nagios.
 *
 * SNMP4Nagios is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * SNMP4Nagios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SNMP4Nagios; if not, write to the
 *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA
 */

#ifndef UTILITIES_H
#define UTILITIES_H


/**********************/
/* Exported functions */

/*
 * Writes the short standard text about copyright and version.
 */
void HelpCopyright ();

/*
 * Writes the short standard text about copyright and version (that is:
 * calls HelpCopyright()) and exits with STATE_OK.
 */
void Version ();

/*
 * Writes that a usage error occured, the synopsis of the parameters
 * and exits with STATE_UNKNOWN.
 *
 * param aMsg: Details about the usage error.
 */
void Usage ( char* aMsg );

/*
 * Write the help text for the standard options.
 */
void HelpStandardOptions ();

#if(WITH_RRD)
/*
 * Write the help text for the performance data logging and plotting options.
 */
void HelpPerfDataOptions ();
#endif /* WITH_RRD */

/*
 * Writes the help text for the ways to set the community string.
 */
void HelpCommunity ();

/*
 * Tries to interpret the arguments and assign the appropriate values to
 * the global variables. This also calls ChkOptSpecificParams().
 *
 * Note that host is set to a newly allocated string which should be freed
 * by the caller.
 */
void ReadArgs ( int argc, char** argv );

#endif /* UTILITIES_H */
